#!/usr/bin/perl
#
# RSS 2 JSON
# Convert RSS (and Atom) feeds to JSON objects.
# By John Resig (http:/ejohn.org/)
#
# This work is licensed under a Creative Commons Attribution 2.5 License.
#
# This script requres that the following modules be installed:
#  - LWP
#  - XML::Feed
#  A custom version of the XML::RSS module is included with this 
#  distribution and should be named XML::RSS3.
#

use CGI;
use LWP::Simple;
use XML::Feed;
use XML::Feed::RSS;

$XML::Feed::RSS::PREFERRED_PARSER = "XML::RSS3";

my $cgi = new CGI();
my $data;
my $call = $cgi->param('callback');
my $url = $cgi->param('url');
my $nurl = $url;
$nurl =~ s/[^a-zA-Z0-9.]//g;
my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
my $f = "cache/$hour\-$mday\-$mon\-$year\-$nurl";

print "Content-type: text/javascript\n\n";

if ( !$url ) {
	exit(0);
} elsif ( -e $f ) {
	open( F, $f );
	$data = join('',<F>);
	close(F);
} else {
	$data = get($url);
	open( F, ">$f" );
	print F $data;
	close(F);
}

my $f = XML::Feed->parse(\$data);

my $c = {};
foreach ( qw(title link description author) ) {
	$c->{$_} = cl( $f->$_() );
}
$c->{modified} = ct( $f->modified() );

print "$call(" if ( $call );
print qq~{
  title: $c->{title},
  link: $c->{link},
  description: $c->{description},
  author: $c->{author},
  modified: $c->{modified},
  entries: [
~;
my @e = $f->entries;
for ( my $i = 0; $i <= $#e; $i++ ) {
	my $c = {};
	foreach ( qw(title link author id) ) {
		$c->{$_} = cl( $e[$i]->$_() );
	}
	$c->{content} = cl( $e[$i]->content->body );
	my @t = @{$e[$i]->category()};
	$c->{category} = "[" . join(',', map{cl($_)}@t ) . "]";
	$c->{issued} = ct( $e[$i]->issued() );
	$c->{modified} = ct( $e[$i]->modified() );
	print qq~    {
      title: $c->{title},
      link: $c->{link},
      content: $c->{content},
      author: $c->{author},
      category: $c->{category},
      issued: $c->{issued},
      modified: $c->{modified},
      id: $c->{id}
    }~;
	print ",\n" if ( $i != $#e );
}
print qq~
  ]
}~;

print ");" if ( $call );

sub cl {
	my $a = $_[0];
	$a = $a->[0] if ( ref($a) eq 'ARRAY' );
	$a =~ s/'/\\'/g;
	$a =~ s/\r?\n/ /g;
	$a = "'$a'";
	$a;
}

sub ct {
	$_[0] ?
		( $call ?
			"new Date('" . $_[0]->ymd . ' ' . $_[0]->hms . "')" :
			"'" . $_[0]->ymd . ' ' . $_[0]->hms ."'" ) :
		"null";
}
